@@ -43,7 +43,7 @@ from .models import CleanupAction, GetAction, GetAndCleanUp
 from .typing_helpers import lru_cache_property
 from .urn import URN
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 class CloudWandererBoto3ResourceFactory:
@@ -295,6 +295,8 @@ class CloudWandererBoto3Service:
     def resource_summary(self) -> List["ResourceSummary"]:
         """Return a summary of resource types in this service."""
         summaries = []
+        service_model = self.boto3_service.meta.client.meta.service_model
+        service_friendly_name = service_model.metadata["serviceId"]
 
         for resource_type in self.resource_types:
             resource = self._get_empty_resource(resource_type)
@@ -304,6 +306,8 @@ class CloudWandererBoto3Service:
             summaries.append(
                 ResourceSummary(
                     resource_type=resource_type,
+                    resource_friendly_name=resource.boto3_resource.meta.resource_model.name,
+                    service_friendly_name=service_friendly_name,
                     secondary_attribute_names=resource.secondary_attribute_names,
                     subresource_types=resource.subresource_types,
                 )
@@ -535,7 +539,7 @@ class CloudWandererBoto3Resource:
         id_members = [x.name for x in self.boto3_resource.meta.resource_model.identifiers]
         resource_ids = []
         for id_member in id_members:
-            id_part = getattr(self.boto3_resource, id_member)
+            id_part = str(getattr(self.boto3_resource, id_member))
             if id_part.startswith("arn:"):
                 id_part = "".join(id_part.split(":")[5:])
             resource_ids.append(id_part)
@@ -748,5 +752,7 @@ class ResourceSummary(NamedTuple):
     """A summary of a resource's subresource types and secondary attribute names."""
 
     resource_type: str
+    resource_friendly_name: str
+    service_friendly_name: str
     subresource_types: List[str]
     secondary_attribute_names: List[str]
